'' =================================================================================================
''
''   File....... sk6812_demo.spin
''   Purpose.... sk6812rgbw and sk6812/ws2812b demonstration program

''               Based on code by Jon McPhalen

''   Edited by.. Darren Payson
''               Copyright (c) 2016-2018 Darren Payson                  
''               -- see below for terms of use
''   E-mail..... darren@paysonplace.net
''   Started.... 4_24_2016
''   Updated.... 
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x                                     
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us
  

con { io pins }

  RX1  = 31                                                     ' programming / terminal
  TX1  = 30
  
  SDA  = 29                                                     ' eeprom / i2c
  SCL  = 28

  LEDS = 0                                                      ' LED tx pin


con

  STRIP_LEN = 16                                                ' # of LEDs in strip
            
  WW        = 0                                                 ' White LEDS? 1 for sk6812RGBW strips, 0 for sk6812 and ws2812b strips 
  
obj

' main                                                          ' * master Spin cog
  time  : "jm_time"                                             '   timing and delays
  io    : "jm_io_basic"                                         '   essential io
  strip : "sk6812"                                           ' * SK2812RGBW LED driver
                                                                 
' * uses cog when loaded                                         
                                                                 

var


dat

  Chakras       long    strip#RED, strip#ORANGE, strip#YELLOW 
                long    strip#GREEN, strip#BLUE, strip#INDIGO


pub main | pos

  setup  
    
  repeat 3
    color_wipe($20_00_00_00, 25)
    color_wipe($00_20_00_00, 25)  
    color_wipe($00_00_20_00, 25)
    if WW == 1  
     color_wipe($00_00_00_20, 25)

  repeat
    repeat pos from 0 to 255
      strip.set_all(strip.wheelx(pos, $40))                     ' 1/4 brightness
      time.pause(20)
      

pub color_chase(p_colors, len, ms) | base, idx, ch

'' Performs color chase 

  repeat base from 0 to len-1                                   ' do all colors in table
    idx := base                                                 ' start at base
    repeat ch from 0 to strip.num_pixels-1                      ' loop through connected leds
      strip.set(ch, long[p_colors][idx])                        ' update channel color 
      if (++idx == len)                                         ' past end of list?
        idx := 0                                                ' yes, reset
   
    time.pause(ms)                                              ' set movement speed


pub setup                                                        
                                                                 
'' Setup IO and objects for application                          
                                                                 
  time.start                                                    ' setup timing & delays
                                                                 
  io.start(0, 0)                                                ' clear all pins (master cog)

  strip.start_b(LEDS, STRIP_LEN, 2_0, WW)                       ' start led driver (2ms reset period)
  strip.off
                                                                 
   
con

  ' Routines ported from C code by Phil Burgess (www.paintyourdragon.com)


pub color_wipe(rgbw, ms) | ch

'' Sequentially fills strip with color rgbw
'' -- ms is delay between pixels, in milliseconds

  repeat ch from 0 to strip.num_pixels-1 
    strip.set(ch, rgbw)
    time.pause(ms)


pub rainbow(ms) | pos, ch

  repeat pos from 0 to 255
    repeat ch from 0 to strip.num_pixels-1
      strip.set(ch, strip.wheel((pos + ch) & $FF))
    time.pause(ms)
    

pub rainbow_cycle(ms) | pos, ch 

  repeat pos from 0 to (255 * 5)
    repeat ch from 0 to strip.num_pixels-1
      strip.set(ch, strip.wheel(((ch * 256 / strip.num_pixels) + pos) & $FF))
    time.pause(ms)
    

dat { license }

{{

  Copyright (c) 2016-2018 Darren Payson 

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}